#!/usr/bin/env python

import sys
import gzip
import os
import sys
import pysam

target_dir = 'data'

if not os.path.isdir(target_dir):
    os.makedirs(target_dir)

_cache_bc = {}
_cache_all_rbcs = {}
def read_cache_bc_from_fastq(fastqname):
    global _cache_bc, _cache_all_rbcs
    _cache_bc = {}

    print "read -> bc mapping from:", fastqname
    sys.stdout.flush()
    fin = gzip.open(fastqname, "rt")
    rid = fin.readline()
    while rid:
        assert rid[0] == '@'
        seq = fin.readline()
        qt = fin.readline()
        assert qt[0] == '+'
        q = fin.readline()
   
        rid = rid.rstrip('\n\r')
        rid = rid[1:]
        # split by '/' to get [read_id] / [part:where:last:item:is:RNDBARCODE]
        rids = rid.split('/', 1)
        assert len(rids) == 2
        rid = rids[0]
        rbc = rids[1].split(':')[-1]
        rbc = _cache_all_rbcs.setdefault(rbc, rbc)
        assert _cache_bc.setdefault(rid, rbc) == rbc

        rid = fin.readline()
    fin.close()
    print "info on %s reads" % len(_cache_bc)
    sys.stdout.flush()

def get_bc_from_dictionary(rname, rbc_indxs):
    global _cache_bc
    bc = _cache_bc[rname]
    return "".join([bc[i] for i in rbc_indxs])

def get_bc_from_header(rname, rbc_indxs):
    bc = rname.rsplit(':', 1)[1]
    return "".join([bc[i] for i in rbc_indxs])

def get_bc_from_nowhere(rname, rbc_indxs):
    return ""

# prepare read names (rname) to include random barcodes
for bam_fname, fastq_fname, get_rndbcode, rndbcode_indexes in [
    ('PTB_CLIP.bam', '', get_bc_from_nowhere, []),
    ('PTB_rep3.bam', '', get_bc_from_header, [0, 1, 2, 7, 8]),
    ('SRSF3_rep2.bam', 'SRSF3_rep2.fastq.gz', get_bc_from_dictionary, [0, 1, 4, 5]),
    ('SRSF4_rep1.bam', 'SRSF4_rep1.fastq.gz', get_bc_from_dictionary, [0, 1, 4, 5]),
    ('TIAL1_rep4.bam', 'TIAL1_rep4.fastq.gz', get_bc_from_dictionary, [2, 3, 4]),
    ('U2AF65.bam', 'U2AF65.fastq.gz', get_bc_from_dictionary, [3, 4, 5, 6]),
    ('eIF4A3_CLIP.bam', '', get_bc_from_nowhere, []),
    ('eIF4A3_rep5.bam', '', get_bc_from_header, [0, 1, 2, 7, 8]),
    ('hnRNPL.bam', '', get_bc_from_header, [0, 1, 2, 7, 8]),
    ]:

    print "processing:", bam_fname
    sys.stdout.flush()
    bam_in_fname = 'raw_data/%s' % bam_fname
    bam_out_fname = '%s/%s' % (target_dir, bam_fname)

    if os.path.isfile(bam_out_fname):
        print "skipping, processed bam already present:", bam_out_fname
        continue

    if fastq_fname:
        fastq_fname = 'raw_data/%s' % fastq_fname
        read_cache_bc_from_fastq(fastq_fname)
        assert get_rndbcode == get_bc_from_dictionary
    else:
        assert get_rndbcode <> get_bc_from_dictionary

    in_bam = pysam.Samfile(bam_in_fname, 'rb')
    out_bam = pysam.Samfile(bam_out_fname, 'wb', template=in_bam)

    for r in in_bam:
        rname = r.qname
        bc = get_rndbcode(rname, rndbcode_indexes)
        if bc:
            r.qname = "%s:rbc%s" % (rname, bc)
        out_bam.write(r)
    out_bam.close()
    in_bam.close()

print "DONE"

